const getTrainedSite = require('./build-shopping-cart/trained-sites.js');
const { StashKeys } = require('../support/constants.js');
const cartItemReportData = require('../support/cart-item-report-data.js');


const ScopedStashName = 'diffPopulatedCartWorkflow';


function diffPopulatedCart(runner) {
  const { kahba, prefab } = runner;

  return kahba.if(kahba.runSync(() => {  // if custom workflow exists use that to navigate to the cart and report on accuracy, else show report and end
      const trainedSite = _getTrainedSite(kahba.stash);
      return !!(trainedSite && trainedSite.itemsInCart); // whether or not custom workflow exists for inspecting shopping cart for current site
    }), _inspectTrainedCartWorkflow(runner), _showUntrainedReportWorkflow(runner));
}

function _inspectTrainedCartWorkflow(runner) {
  const { kahba, prefab } = runner;
  const getScopedStash = () => kahba.task().dataStash(ScopedStashName);

  return kahba.during_(kahba.runSync(() => !getScopedStash().done),
    kahba.waterfall([


      kahba.runUntilFinal((result={}) => {
        const productList = _getProductListForSelectedSite(kahba.stash);
        prefab.contentPage.notify('See Wranggle popup for inspection of cart', { removeIfPopupOpens: true });
        prefab.reportTable(cartItemReportData(productList, result.itemsInCart), {
          title: 'Verify Items In Cart Are Correct:',
          segmentId: 'cartDiff',
          sortScore: 100,
        });
        return prefab.sendConfirmationPrompt('Finished adding or removing Items?', {
          cancelButton: false,
        });
      }),

    ])
  );

}

function _showUntrainedReportWorkflow(runner) {
  const { kahba, prefab } = runner;
  return kahba.runSync(() => { // else no custom workflow so display report and end
    const itemsInCart = kahba.stash[StashKeys.ManuallyAddedItems]; // what user reported during manuallyBuildCart
    _showCartDiff(prefab, _getProductListForSelectedSite(kahba.stash), { itemsInCart });
  });
}

function _showCartDiff(prefab, desiredProductList, itemsInCart) {
  prefab.contentPage.notify('See Wranggle popup for inspection of cart', { removeIfPopupOpens: true });
  prefab.reportTable(cartItemReportData(desiredProductList, itemsInCart), {
    title: 'Verify Items In Cart Are Correct:',
    segmentId: 'cartDiff',
    sortScore: 100,
  });
}

function _inspectCart(runner) {
  const { kahba, prefab } = runner;
  return kahba.taskFactorySync(() => {
    const selectedSite = kahba.stash[StashKeys.SelectedSite];
    const trainedSite = getTrainedSite(selectedSite);
    const itemsInCartWorkflow = trainedSite && trainedSite.itemsInCart;
    if (itemsInCartWorkflow) {
      return itemsInCartWorkflow(runner, {
        productList: _getProductListForSelectedSite(kahba.stash)
      });
    } else {
      return () => Promise.resolve({ itemsInCart: kahba.stash[StashKeys.ManuallyAddedItems] });
    }
  });
}


function _getSelectedSite(stash) {
  return stash[StashKeys.SelectedSite];
}

function _getTrainedSite(stash) {
  return getTrainedSite(_getSelectedSite(stash));
}

function _getProductListForSelectedSite(stash) {
  return (stash[StashKeys.AllProductsByHost] || {})[_getSelectedSite(stash)];
}


module.exports = diffPopulatedCart;
