const _ = require('lodash');



function cartItemReportData(desiredItems, cartItems=[]) {
  const rows = [];
  const desiredByUrl = _.keyBy(desiredItems, 'url');

  cartItems && cartItems.forEach(item => {
    const url = item.url;
    const desiredItem = url && desiredByUrl[url];
    const tableRow = {
      product: _productLinkOrName(url, item.name || desiredItem.name),
      quantity: item.quantity || 'n/a',
      status: null
    };
    if (!desiredItem) {
      tableRow.status = '<strong class="text-danger">Unexpected item!</strong>';
      tableRow.sortNdx = 1;
    } else if (item.manuallyAddedStatus) {
      tableRow.quantity = desiredItem.quantity;
      tableRow.status = `<strong class="text-success">${ item.manuallyAddedStatus || 'Skipped' }</strong>`;
      tableRow.sortNdx = 3;
    } else if (parseInt(item.quantity) !== parseInt(desiredItem.quantity)) {
      tableRow.status = `<strong class="text-danger">Wrong quantity, expecting ${ desiredItem.quantity }</strong>`;
      tableRow.sortNdx = 2;
    } else {
      tableRow.status = '<strong class="text-success">Looks ok</strong>';
      tableRow.sortNdx = 3;
    }
    rows.push(tableRow);
  });

  const missingProductUrls = _.differenceBy(Object.keys(desiredByUrl), _.map(cartItems, 'url'));
  missingProductUrls.forEach(url => {
    const item = desiredByUrl[url];
    rows.push({
      product: _productLinkOrName(url, item.name),
      quantity: item.quantity,
      status: '<strong class="text-danger">Missing!</strong>',
      sortNdx: 2
    })
  });
  const tableData = _.sortBy(rows, [ 'sortNdx', 'name' ]);
  tableData.forEach(row => delete row.sortNdx);
  return tableData;
}

function _productLinkOrName(url, name) {
  name = name || 'Unknown Item';
  return url ? `<a href="${ url }">${ name }<a/>` : name;
}

module.exports = cartItemReportData;