const _ = require('lodash');
const parse = require('csv-parse/lib/sync')
const urlSupport = require('./url-support.js');


/**
 *
 * @param rawCsvBody: csv content as string containing product data (see parseCsv below for column order)
 *  Rows with a missing or invalid URL are skipped (such as the first row containing headers, blank lines, etc.)
 *  Rows with a non-empty 'purchasedAt' value are skipped (allowing the same csv data to include historical data)
 */
function parseCsvAndGroupProductsByHostname(rawCsvBody) {
  const productData = parseCsv(rawCsvBody);
  return groupProductsByHostname(productData);
}


/**
 *
 * @param rawCsvBody: csv as string, where each row contains 5 column-separated columns in the following order:
 *    0: "purchasedAt"
 *    1: "name"
 *    2: "quantity"
 *    3: "url"
 *    4: "notes"
 */
function parseCsv(rawCsvBody='') {
  const rows = parse(rawCsvBody);
  return rows.map((row, lineNumber) => {
    const [ purchasedAt, name, quantity, url, notes ] = row || [];
    return { lineNumber, purchasedAt, name, quantity, url, notes };
  });
}


/**
 * Groups product data by hostname, filtering items already purchased or with invalid URLs
 * @param productList
 * @returns {*}
 */
function groupProductsByHostname(productList) {
  const res = _.groupBy(productList, itemInfo => {
    const url = itemInfo.url;
    if (url && urlSupport.looksLikeAbsoluteUrl(url)) {
      return urlSupport.shortDomain(url); // host, dropping "www." when present too
    } else {
      return 'ignore';
    }
  });
  delete res.ignore;
  return res;
}


module.exports = {
  parseCsvAndGroupProductsByHostname,
  groupProductsByHostname,
}