const _ = require('lodash');
const { StashKeys } = require('../../support/constants.js');
const getTrainedSite = require('./trained-sites.js');
const manuallyBuildCart = require('./manually-build-cart.js');


function buildShoppingCart(runner) {
  const { kahba } = runner;
  // At the time this function is called, Kahba hasn't started running and kahba.stash.selectedSite has not yet been set by the user.
  //   We use kahba.taskFactory to supply a site-specific populateCart task during runtime
  return kahba.taskFactorySync(() => {
    const selectedSite = kahba.stash[StashKeys.SelectedSite];
    const productList = (kahba.stash[StashKeys.AllProductsByHost] || {})[selectedSite];
    const trainedSite = getTrainedSite(selectedSite);
    const populateCartWorkflow = trainedSite && trainedSite.populateCart;
    if (populateCartWorkflow) {
      return populateCartWorkflow(runner, productList);
    } else {
      return manuallyBuildCart(runner, productList);
    }
  });
}


module.exports = buildShoppingCart;
