const _ = require('lodash');
const { StashKeys } = require('../../support/constants.js');

/**
 *
 * @param runner
 * @param productList: Array of items to purchase, with each item an object of { name, url, quantity, notes } for the product.
 * @returns {*}
 */
function manuallyBuildCart(runner, productList) {
  const { kahba, prefab } = runner;

  return kahba.eachSeries(productList, // normally you'd pass eachSeries a task that resolves to a collection for looping but we happen to have the productList handy
    kahba.conveyor([
      runner.visitThePage({ get: 'url' }).opts({ forceResult: true }), // the "get" option tells visitThePage to use lodash _.get to pick the URL out of passed object arguments as it searches (so it uses productData.url here)

      // we use kahba.runUntilFinal so the user can navigate freely while working on their shopping cart.
      kahba.runUntilFinal((productData, conveyorData, next) => { // kahba.conveyor includes the current productData item and its own conveyorData object
        const qty = productData.quantity || 1;
        document.title = `Buy ${ qty }`;
        const promptOpts = {
          description: _popupConfirmationPromptDescription(productData, conveyorData.error),
          trueButton: 'Done',
          falseButton: 'Skip',
          contentPage: true
        };
        prefab.sendConfirmationPrompt(`Add ${ qty } to cart: ${ productData.name || 'this item' }`, promptOpts, (err, isConfirmed) => {
          kahba.stash[StashKeys.ManuallyAddedItems] = kahba.stash[StashKeys.ManuallyAddedItems] || [];
          kahba.stash[StashKeys.ManuallyAddedItems].push(Object.assign({}, productData, {
            manuallyAddedStatus: !err && isConfirmed ? 'Added by me' : 'Skipping item'
          }));
          next();
        });
      }),

    ]));
}

function _popupConfirmationPromptDescription(productData, navError) {
  const details = `<p><a href="${ productData.url }">Product</a> notes:<br/> ${ productData.notes || 'none'}</p>`;
  if (navError) {
    return `<div><p class="lead">Take care! URL provided might be incorrect.</p>${ details }</div>`;
  } else {
    return details;
  }
}


module.exports = manuallyBuildCart;
