const _ = require('lodash');
const $ = require('jquery');


function populateCart(runner, productList) {
  const { kahba } = runner;
  return kahba.eachSeries(productList,
    kahba.waterfall([
      runner.visitThePage({ get: 'url' }),
      kahba.if('button:visible:contains("Stock")', kahba.error()), // fails faster if it detects item is out of stock (else the wait for add-to-cart button would time out)
      // note: not setting quantity here--something that can be done more easily on the cart checkout page.
      runner.click('button:visible:contains("Add to Cart")', { expectNavigation: false }), // add-to-cart uses XHR so explicitly setting expectNavigation option to speed things up, otherwise the click method needs to guess about navigation and might time out
      runner.wait('.modal-content:visible,button:visible:contains("Continue")').opts({ timeout: 1000 }) // this wait isn't essential so if the selectors break, it doesn't slow things down too much
    ], productErr => {
      console.warn('Possible problem adding product', location.href, productErr);
      return []; // suppresses errors. Makes best attempt to add each, and then reports final results to the user after inspecting the cart page
    }));
}


function itemsInCart(runner, opts={}) {
  const { kahba } = runner;
  // todo: check if already on cart page. kahba.if
  const productsExpected = opts.productList;

  // Instead of explicitly setting our result on kahba.stash as before, we streamline things a bit this time using a kahba.waterfall.
  // The waterfall uses the result of its last step as its final result, so the final cart dat will be available as an argument
  // to the parent control flow. (In contrast, kahba.series/async.series would return an array containing the result of each step.)
  return kahba.waterfall([
    runner.visit('https://www.walmart.com/cart', {
      expect: '.cart-content:visible' // the "expect" option waits on the page until its conditions are met (or it times out)
    }),

    kahba.runSync(() => {
      const cartData = _.map($('.cart-item-row:visible select:visible'), (el) => {
        const $quantitySelector = $(el);
        const $row = $quantitySelector.closest('.cart-item-row'); // walmart has .cart-item-row inside another .cart-item-row, might be unstable ground, so finding the actual row using quantity selector and expanding up
        const quantity = parseInt($quantitySelector.val());
        const psuedoId = $.trim($row.find('a[data-us-item-id]').attr('data-us-item-id')); // The walmart.com cart page does not include the product url in the DOM, so identifying it here using "psuedoId" attrib data
        const name = $.trim($row.find('.cart-item-name').text());

        const matchingItem = productsExpected.find(desiredItemData => desiredItemData.url.indexOf(psuedoId) > 0 || desiredItemData.name === name);
        const url = matchingItem && matchingItem.url;
        return { psuedoId, quantity, name, url };
      });
      return { itemsInCart: cartData }; // runSync converts this to an async callback result for you
    })
  ]);
}


module.exports = {
  populateCart,
  itemsInCart
};
