const getTrainedSite = require('./build-shopping-cart/trained-sites.js');
const { StashKeys } = require('../support/constants.js');
const cartItemReportData = require('../support/cart-item-report-data.js');


function diffPopulatedCart(runner) {
  const { kahba, prefab } = runner;

  return kahba.conveyor([
    () => {  // copying info out of stash into conveyorData as a minor convenience
      const selectedSite = kahba.stash[StashKeys.SelectedSite];
      return Promise.resolve({
        selectedSite,
        productList: (kahba.stash[StashKeys.AllProductsByHost] || {})[selectedSite]
      });
    },

    kahba.taskFactorySync((conveyorData) => {
      const checkItemsInCartWorkflow = (getTrainedSite(conveyorData.selectedSite) || {}).itemsInCart;
      if (checkItemsInCartWorkflow) {
        return checkItemsInCartWorkflow(runner, { productList: conveyorData.productList });
      } else {
        return () => Promise.resolve({ itemsInCart: kahba.stash[StashKeys.ManuallyAddedItems] });
      }
    }),
    
    kahba.runUntilFinal(conveyorData => {
      prefab.contentPage.notify('See Wranggle popup for inspection of cart', { removeIfPopupOpens: true });
      prefab.reportTable(cartItemReportData(conveyorData.productList, conveyorData.itemsInCart), {
        title: 'Verify Items In Cart Are Correct:',
        segmentId: 'cartDiff',
        sortScore: -1,
      });
      return prefab.sendConfirmationPrompt('Finished adding or removing Items?', {
        cancelButton: false,
      });
    }),
    
  ]).subtaskOpts({ deepParams: 1 }); // kahba.conveyor is meant for immediate nesting (eg, inside eachSeries) but setting deepParams 1 isolates it from any params passed in
}

// TODO: include more advanced functionality... perhaps:
//   1) command events sent by UI (to redo/refresh cart check)
//   2) use of kahba.race and/or kahba.during_?
//   3) use named/marked options instead of using kahba.stash?


module.exports = diffPopulatedCart;
