const { StashKeys } = require('../support/constants.js');
const getTrainedSite = require('./build-shopping-cart/trained-sites.js');

function selectDesiredStore(runner) {
  const { kahba, prefab } = runner;

  return kahba.runUntilFinal(() => {
    const cb = kahba.task().getCallback();
    const productListByDomain = kahba.stash[StashKeys.AllProductsByHost];
    const storeDomains = Object.keys(productListByDomain).sort();
    if (storeDomains.length === 0) {
      return cb({
        errorCode: 'EmptyProductList',
        userMessage: 'Nothing to buy!'
      });
    }
    const choices = { none: 'Select store' };
    _.each(storeDomains, domain => {
      const trainedSite = getTrainedSite(domain);
      choices[domain] = `${ domain } (${ trainedSite && trainedSite.populateCart ? 'partially automated' : 'manual/guided' })`;
    });
    prefab.sendSelectPrompt(choices, {
      title: 'Select store for checkout:',
      cancelButton: true,
      value: storeDomains.length === 1 ? storeDomains[0] : 'none',
    }, (err, choice) => {
      kahba.stash[StashKeys.SelectedSite] = choice;
      if (!choice || choice === 'none') {
        err = err || {
          errorCode: 'CancelledByUser',
          userMessage: 'Cancelled. See you later.',
        }
      }
      cb(err);
    });
  });
}


module.exports = selectDesiredStore;
